import os
import sys
sys.dont_write_bytecode = True
from .shellcolor import scolor
from . import common,startinstall
import datetime
import argparse
from pprint import pprint
import re,socket
import configparser

def read_confgdata(configfile):
    data=dict()    
    if os.path.isfile(configfile):
        try:
            parser = configparser.ConfigParser()
            with open(configfile,'r') as stream:
                parser.read_string("[top]\n" + stream.read()) 
            data=dict(parser['top'])
        except Exception as e:
            log(str(e))
            data=dict()
            pass
    return data

def findos():
    data=dict()
    os_versions='/etc/os-release'
    configs=read_confgdata(os_versions)        
    if  bool(configs):
        try:
            soanmes=configs[str('PRETTY_NAME').lower()]
            if str(soanmes).startswith('"'):
                soanmes=soanmes[1:]
            if str(soanmes).endswith('"'):
                soanmes=soanmes[:-1]            
            if soanmes.startswith(('Ubuntu')):
                data["os"]='Ubuntu'
                ubuname=configs[str('UBUNTU_CODENAME').lower()]
                if str(ubuname).startswith('"'):
                    ubuname=ubuname[1:]
                if str(ubuname).endswith('"'):
                    ubuname=ubuname[:-1]
                data["name"]=ubuname
                data["pretty_name"]=soanmes
            elif soanmes.startswith(('Red','Rocky','AlmaLinux','CloudLinux')):
                data["os"]='RHEL'
                rhname=configs[str('ID').lower()]
                if str(rhname).startswith('"'):
                    rhname=rhname[1:]
                if str(rhname).endswith('"'):
                    rhname=rhname[:-1]
                data["name"]=rhname
                data["pretty_name"]=soanmes
        except Exception as e:
            print(str(e))
            data=dict()
            pass    
    return data

def banner():
    now = datetime.datetime.now()
    sys.stdout.write("Copyright (C) 2023-"+str(now.year)+", Eenos.com \n")
    sys.stdout.write("Eenos High Performance Hosting Control Panel\n")
    sys.stdout.write(scolor.green)
    sys.stdout.write('''
 _  _       _   __ 
|_ |_ |\ | / \ (_  
|_ |_ | \| \_/ __) 
''')
    sys.stdout.write(scolor.end)
    sys.stdout.write("\nEenos auto installer tool\n")
   
def help():
    banner()
    print('''
About:
Eenos is a modern high performance hosting control panel.
It is a replacement for your traditional hosting panels.
Eenos is developed by a team of System Engineers on works on
data centers and web hosting companies since the late 90's.
''')

def validate_emailname(value):
    data=value.split('@')    
    if len(data) ==2:
        name=data[0]
        domain=data[-1]
        pattern=r"^[a-zA-Z0-9_]([a-z0-9._%+-]{0,63}|[A-Za-z0-9._%+-]{0,63})$(?<![._%+-])"
        dpattern=r"^(([a-zA-Z0-9]|\*\.[a-zA-Z0-9])|[a-zA-Z0-9][a-zA-Z0-9-]){0,254}[a-zA-Z0-9](?:\.[a-zA-Z0-9]{2,})+$"
        if re.match(pattern,name) and re.match(dpattern,domain):        
            return value
        else:
            raise argparse.ArgumentTypeError("Invalid email address")
    else:
        raise argparse.ArgumentTypeError("Invalid email address")        

def validate_webserver(value):
    ws=['nginx','httpd','ols','olsvarnish','nginxhttpd','nginxvarnish','httpdvarnish','nginxhttpdvarnish','nginxolsvarnish']
    if value in ws:
        return value
    else:
         raise argparse.ArgumentTypeError("Invalid web server type entered")
def options(default=False):
    parser = argparse.ArgumentParser(prog='setup.py',description=banner(),usage='%(prog)s [options] install ',formatter_class=argparse.RawTextHelpFormatter,add_help=False)
    parser._optionals.title='Option(s)'
    parser._positionals.title='Required'
    parser.add_argument("install", type=str,help="Install Eenos Hosting Control Panel")   
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument('--atos',action='store_true',help='Accept Eenos Software License(EPL) and Service Level Agreements\nEenos License (EPL) : https://eenos.com/epl/\nService Level Agreements : https://eenos.com/terms-and-conditions/\nWARNING !!! : You are not allowed to use eenos \nwithout accepting the EPL and SLA\n ')    
    parser.add_argument('--email',metavar="contact_email_address",type=validate_emailname, help='Server contact email address\nDefault : eenos@'+str(socket.gethostname())+'\n ')
    parser.add_argument('--ns',metavar="nameservers",type=str,default="change-me@no-ns.com,change-me@no-ns.net",  help='Comma separated list of default  nameservers\nDefault: change-me@no-ns.com,change-me@no-ns.net\n ')
    parser.add_argument('--setupwebserver','-sw',metavar="webserver_type",type=validate_webserver, default='nginxhttpd', help='''Combination of web server you like to use 
Choose one , By default  we use nginxhttpd
nginx             , Install nginx only web server
httpd             , Install apache httpd web server
ols               , Install OpenLiteSpeed web server
olsvarnish        , install OpenliteSpeed and varnish cache
nginxhttpd        , Install nginx and apache httpd
nginxvarnish      , Install nginx with varnish cache
httpdvarnish      , Install httpd with varnish cache
nginxhttpdvarnish , Install nginx and httpd with varnish cache
nginxolsvarnish   , Install nginx and OpenLiteSpeed with varnish cache
    ''')
    parser.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS,help='Show this help message and exit.\n ')
    args=None
    if default:
        parser.print_help()
        sys.exit(1)
    else:       
        try:
            args = parser.parse_args()    
        except:  
            # parser.print_help()          
            sys.exit(0)        
    if args.install and args.install == 'install':
        if not args.atos:
            args.atos=False
        if not args.email:
            args.email='eenos@'+str(socket.gethostname())        
        return vars(args)
    else:
        parser.print_help()
        sys.exit(0)

def  force_atos():
    print('''You may need to  read and accept the following :
Eenos Software License (EPL), Read from: https://eenos.com/epl/
Service Level Agreements (SLA), Read from: https://eenos.com/sla/''')    
    tos='n'
    try:
        tos=input("Do you agree the EPL & SLA (y/n)?: ")
    except KeyboardInterrupt:
        print("Aborting installation")
        tos='n'
        sys.exit(1)
    if tos == 'y' or tos == 'Y' or tos == 'yes' or tos == "Yes":
        return True
    else:
        return False

def stage_one(configure):
    os_support=False
    print("Checking Operating System requirement .. ")
    odata=findos()
    pprint(odata)
    if odata['os']== 'Ubuntu':
        print("Found supporting Operating System : %s%s%s" %(scolor.green,odata['pretty_name'].strip('"'),scolor.end))
        odata['version']=odata['name']
        os_support=True
    elif odata['os']== 'RHEL':
        print("Found supporting Operating System : %s%s%s" %(scolor.green,odata['pretty_name'].strip('"'),scolor.end))
        odata['version']=odata['name']
        os_support=True
    else:
        print("%sUnsupported Operating System%s" %(scolor.warning,scolor.end)) 
        sys.exit(0)
    if os_support:
        startinstall.start(odata,configure)
    # if os.path.isfile('/etc/os-release'):
    #     data=common.get_os('/etc/os-release')
    #     if bool(data) and 'ID' in data and 'VERSION_CODENAME' in data and 'VERSION' in data:            
    #         if data['ID'] == 'ubuntu' and data['VERSION_CODENAME'] == 'jammy':
    #             odata['os']='ubuntu'
    #             odata['version']='jammy'
    #             print("Found supporting Operating System : %s%s%s" %(scolor.green,data['VERSION'].strip('"'),scolor.end))
    #             os_support=True
    #         elif data['ID'] == 'ubuntu'
    #         else:
    #             print("%sUnsupported Operating System%s" %(scolor.warning,scolor.end))                
    #             sys.exit(0)
    # if os_support:
    #     startinstall.start(odata,configure)
        
def run():
    if  os.geteuid() == 0:
        if not sys.argv[1:]:
            options(default=True)        
            sys.exit(0)
        else:
            cmd=options()
            # pprint(cmd)
            if 'install' in cmd and 'atos' in cmd:
                if cmd['atos'] == False:
                    atos=force_atos()
                else:
                    atos=True
                if atos:
                    stage_one(cmd)
                else:
                    print("Aborting installation")
            else:
                print("Invalid options")   
            
    else:
        print ("Eenos auto installer")
        print("You don't have privilege to run this tool")
        sys.exit(2)
