import os
import sys
from . import common
from .shellcolor import scolor
from pprint import pprint
import ipaddress
import shutil

def create_basic_settings(mainips,configure,log):

    log("Generating default eenos basic settings ..")
    try:
        bs={}
        bs['contact_email']=list()
        bs['contact_email'].append(configure['email'])
        bs['default_home']='/home'
        bs['extra_home']=list()
        bs['main_shared_ip4']=mainips['ipv4']
        if 'ipv6' in mainips and bool(mainips['ipv6']):
            bs['main_shared_ip6']=mainips['ipv6']
        else:
            bs['main_shared_ip6']=''
        bs['nameservers']=configure['ns'].split(',')        
        bfile='/var/eenos/preferences/basic.yaml'
        log("Creating : %s" %bfile)
        result=common.writeyaml(bfile,bs)
        log(result)
        # pprint(mainips)
        # pprint(configure)
        # pprint(bs)
    except Exception as e:
        log(str(e))
        log("Unknown issue with basic settings generation , please contact support")
        sys.ext(1)
    

def setup_basic(configure,log):
    setupbasic=False
    log("Finding main ips")   
    cmd='/usr/bin/hostname --all-ip-addresses'
    log("Running : %s" %cmd)
    try:
        rdata=common.cmd_runandreturn(cmd)[0]
        ipdata=rdata.strip().split()
    except Exception as e:
        log(str(e))
        log("Networking issue on this server unable to fetch IP details using : , %s" %cmd)
        log("Aborting installation")
        sys.exit(1)
    mainips={}
    try:    
        for ip in ipdata:
            if ':' in ip:
                mainips['ipv6']=ipaddress.ip_address(ip.strip()).exploded
            else:
                mainips['ipv4']=ip.strip()
        if 'ipv4' in mainips and bool(mainips['ipv4']):
            log("Using main shared IPv4 as : %s%s%s" %(scolor.green,mainips['ipv4'],scolor.end))
            setupbasic=True
            if 'ipv6' in mainips and bool(mainips['ipv6']):
                log("Using main shared IPv6 as : %s%s%s"  %(scolor.green,mainips['ipv6'],scolor.end))
            log("You can change the main shared IPs later from %sEENOS -> WAP -> Basic Settings%s" %(scolor.pink,scolor.end))
        else:
            log("Unable to fetch IPv4 address.  Eenos need and IPv4 address to install")
            log("Aborting installation")
            sys.exit(0)
    except Exception as e:
        log(str(e))
        log("Unable to process main ips ")
        sys.exit(1)    
    if setupbasic:
        create_basic_settings(mainips,configure,log)

def setup_tweak(configure,log):
    log("Configuring basic tweak settings ... ")
    ts={
        'alianrestore': 'off',
        'bandwidthsuspend': 'on',
        'forcessl': 'on',
        'quotasuspend': 'on',
        'resellerbranding': 'off',
        'resellerchain': 'off',
        'resellerpool': 'off',
        'resellerpoolrestore': 'off',
        'resellerschedule': 'off'
        }
    tfile='/var/eenos/preferences/tweaksettings.yaml'
    res=common.writeyaml(tfile,ts)    


def setup_webserver(configure,log):
    log("Configuring web servers : %s%s%s" %(scolor.green,configure['setupwebserver'],scolor.end))
    cmd='/usr/local/eenos/scripts/setupwebserver -i '+configure['setupwebserver']
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def multi_php(configure,log):
    log("Installing some php version ")
    multiphp=['eenos-php82','eenos-php81','eenos-php74']   
    for php in multiphp:
        log("Installing php version : %s" %php)
        pcmd1='/usr/local/eenos/scripts/setupphp -i '+php
        log(pcmd1)
        os.system(pcmd1)
        # common.cmd_live(pcmd1,log)
    pcmd2='/usr/local/eenos/scripts/setupphp -d '+str(multiphp[0])
    pcmd3='/usr/local/eenos/scripts/setupphp -dh suphp'
    log(pcmd2)
    log("Setting default php %s" %multiphp[0])
    os.system(pcmd2)
    # common.cmd_live(pcmd2,log)
    log(pcmd3)
    os.system(pcmd3)
    # common.cmd_live(pcmd3,log)
    ldcmd='/usr/sbin/ldconfig'
    os.system(ldcmd)
    for php in multiphp:
        fpm=php+'-fpm.service'
        log("Restarting php-fpm service : %s" %fpm)
        rcmd='/usr/bin/systemctl restart '+fpm
        os.system(rcmd)
        # common.cmd_live(rcmd,log)
        rstat='/usr/bin/systemctl status '+fpm+' --no-pager --no-block'
        os.system(rstat)        
    result='/usr/local/eenos/scripts/setupphp -l'
    os.system(result)
    # common.cmd_run(result,log)

def setup_dns(log):
    log("Installing bind9 dns server")
    cmd='/usr/local/eenos/scripts/setupdns install=bind'
    log(cmd)
    os.system(cmd)  
    # common.cmd_live(cmd,log)

def setup_eenosd(log):
    log("Installing eenosd web server")
    cmd='/usr/local/eenos/scripts/setupeenosd -i'
    log(cmd)
    os.system(cmd)  
    # common.cmd_live(cmd,log)

def setup_eenos_filemanager(os_data,log):
    log("Installing eenos file manager")  
    if os_data['os']=='RHEL':
        cmd="/usr/bin/dnf -y install eenos-filemanager"
    else:
        cmd="/usr/bin/apt-get -y install eenos-filemanager"
    log(cmd)
    os.system(cmd)

def license_update(log):
    log("Initializing license for eenos")
    lcmd='/usr/local/eenos/bin/eenoslicense -i'
    common.cmd_live(lcmd,log)
    lcmd='/usr/local/eenos/bin/eenoslicense -u'
    common.cmd_live(lcmd,log)
    lcmd='/usr/local/eenos/bin/eenoslicense -c'
    common.cmd_live(lcmd,log)

def setup_mailserver(log):
    log("Installing exim & dovecot mail server")
    cmd='/usr/local/eenos/scripts/setupmailserver --install'
    log(cmd)
    os.system(cmd)  
    # common.cmd_live(cmd,log)

def setup_hostname_ssl(log):
    log("Installing Lets encrypt auto ssl for hostname ")
    cmd='/usr/local/eenos/scripts/hostnamesslinstall'
    log(cmd)
    os.system(cmd) 
    # common.cmd_live(cmd,log)

def setup_ftp(log):
    log("Installing FTP Server")
    cmd='/usr/local/eenos/scripts/setupftp --install'
    log(cmd)
    os.system(cmd) 
    # common.cmd_live(cmd,log)
def setup_mysql_pkgconf(log):
    log("Setting up pkg-config for libpercona")
    pconf='''#Percona or Mysql8 doesn't ship pkg-config
# Eenos just provide it
# copy to /usr/lib/x86_64-linux-gnu/pkgconfig/mysqlclient.pc
prefix=/usr
includedir=${prefix}/include/mysql
libdir=${prefix}/lib/x86_64-linux-gnu

Name: mysqlclient
Description: Percona MySQL client library
Version: 21.2.34
Cflags: -I${includedir} 
Libs: -L${libdir} -lperconaserverclient 
Libs.private: -lpthread -lssl -lcrypto -lresolv -lm -lrt
    '''
    try:
        with open('/usr/lib/x86_64-linux-gnu/pkgconfig/mysqlclient.pc','w') as pc:
            pc.write(pconf)
    except Exception as e:
        log(str(e))
        pass

def setup_mysql(os_data,log):
    log("Installing High performance Percona mysql server")
    if os_data['os'] in ['Ubuntu','ubuntu']:
        setup_mysql_pkgconf(log)
    cmd='/usr/local/eenos/scripts/setupmysql --install'
    log(cmd)
    os.system(cmd)

def setup_cert_bot(log):
    cmd='/usr/local/eenos/scripts/certbotinstall'
    log(cmd)
    os.system(cmd)

def setup_certbot_time(log):
    if os.path.isfile('/usr/local/eenos/templates/systemd/certbot.service') and os.path.isfile('/usr/local/eenos/templates/systemd/certbot.timer'):
        log("Installing certbot auto renewal")
        shutil.copy('/usr/local/eenos/templates/systemd/certbot.service','/lib/systemd/system/certbot.service')
        shutil.copy('/usr/local/eenos/templates/systemd/certbot.timer','/lib/systemd/system/certbot.timer')
        cmd="/usr/bin/systemctl enable certbot.service"
        cmd2="/usr/bin/systemctl enable certbot.timer"
        os.system(cmd)
        os.system(cmd2)
        rcmd="/usr/bin/systemctl restart certbot.service"
        rcmd2="/usr/bin/systemctl restart certbot.timer"
        os.system(rcmd)
        os.system(rcmd2)
        cmd="/usr/bin/systemctl status certbot.timer --no-pager --no-block"
        # common.cmd_run(cmd,log)
        os.system(cmd)

def stage_three(os_data,configure,log):
    log("Starting stage three setup")
    setup_basic(configure,log)
    setup_tweak(configure,log)
    setup_dns(log)
    setup_webserver(configure,log)
    multi_php(configure,log)
    setup_eenosd(log)
    setup_eenos_filemanager(os_data,log)
    license_update(log)
    setup_mailserver(log)    
    setup_ftp(log)
    setup_mysql(os_data,log)
    setup_cert_bot(log)
    # setup_certbot_time(log)
    setup_hostname_ssl(log)
