import os
from pprint import pprint
from . import common,eenosboot
from .shellcolor import scolor
import urllib.request
import sys
import tempfile

def rhel_dnf_setup(log):
    if os.path.isfile('/usr/bin/dnf'):
        log("Starting DNF Setup")
        log("Importing Eenos RPM Keys")
        rpm_key='/usr/bin/rpm --import https://dnf.eenos.com/eenos.asc'
        os.system(rpm_key)
        log("Installing Eenos DNF repository")
        repo_cmd='/usr/bin/dnf -y install https://dnf.eenos.com/eenos-repo-0.1.1-1.noarch.rpm'
        os.system(repo_cmd)
        log("Installing DNF config manager")
        cnfm="/usr/bin/dnf -y install dnf-plugins-core"
        os.system(cnfm)        
        log("Activating CRB repo")
        crb='/usr/bin/dnf config-manager --set-enabled crb'
        os.system(cnfm)
        log("Cleaning the DNF cache")
        clcmd='/usr/bin/dnf clean all'
        os.system(clcmd)
        log("Updating DNF cache")
        dnfup='/usr/bin/dnf -y update'
        os.system(dnfup)
    else:
        log("Unable to setup DNF repository")
        sys.exit(1)
   
    
def eenosd_python(log):
    log("Installing Eenos python distribution  - [ %seenosd-python%s ]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenosd-python"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenoscore(log):
    log("Installing Eenos core package - [%seenoscore%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenoscore"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_pip(log):
    
    # print(tempfile.gettempdir())
    ctmp=str(tempfile.gettempdir())
    tmdir='/root/tmp'
    if not os.path.isdir('/root/tmp'):
        try:
            os.makedirs('/root/tmp')
        except Exception as e:
            log(str(e))
            pass
    os.environ['TMPDIR']=tmdir
    result=0
    try:
        log("Installing Eenos Pips")
        cmd="/usr/local/eenos/bin/pip3 --no-color --no-cache-dir install --extra-index-url https://pip.eenos.com/release/ eenos"
        log(cmd)
        result=os.system(cmd)
    except Exception as e:
        log(str(e))
        log("Error installing python modules. Please make sure /tmp have execute permission at this time")
        sys.exit(0)
    if result > 0:
        log("Error installing python modules. Please make sure /tmp have execute permission at this time")
        log("Then re-run the installer")
        sys.exit(0)
    os.environ['TMPDIR']=ctmp 
    # common.cmd_live(cmd,log)

def eenos_web(log):
    log("Installing Eenos web package - [%seenos-web%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-web-release"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
def eenos_dbs(log):
    log("initializing eenos dbs")
    cmd='/usr/local/eenos/scripts/dbinit'
    os.system(cmd)
def eenos_bfd(log):
    log("Installing eenos brute force detection service - [%seenos-bfd%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-bfd" 
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    log("Initializing bfd db")
    cmd2="/usr/local/eenos/scripts/bfdctl -l"
    log(cmd2)
    os.system(cmd2)
    # common.cmd_live(cmd2,log)
    rcmd='/usr/bin/systemctl restart eenosbfd.service'
    log(rcmd)
    os.system(rcmd)
    # common.cmd_live(rcmd,log)

def eenos_default_html(log):
    log("Setting default html docs")
    cmd="/usr/bin/dnf -y install eenos-default-html"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def eenos_diskusage(log):
    log("Installing eenos disk usage monitor - [%seenos-diskusage%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-diskusage"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_monitor(log):
    log("Installing eenos service monitor - [%seenos-monitor%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-monitor"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_networking(log):
    log("Installing eenos networking service - [%seenos-network%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-network"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_taskd(log):
    log("Installing eenos taskd service - [%seenos_taskd%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-taskd"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def eenos_traffic(log):
    log("Installing eenos bandwidth monitoring service - [%seenos-traffic%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-traffic"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_update(log):
    log("Installing eenos auto updates - [%seenos-update%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/dnf -y install eenos-update"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def cleanup(log):
    log("Cleaning up unused packages")
    cmd="/usr/bin/dnf -y remove httpd postfix php mariadb nginx"
    log(cmd)
    os.system(cmd)

def eenos_jailshell(log):
    log("Installing Eenos JailShell")
    cmd="/usr/bin/dnf -y install eenos-jailshell"
    log(cmd)
    os.system(cmd)

def start_rhel_install(os_data,configure,log):   
    rhel_dnf_setup(log)
    cleanup(log)    
    base_packages='chrony util-linux-user bash-completion psmisc net-tools wget curl perl  bind bind-utils bind-libs git rsync quota cronie whois iputils iptables sqlite'
    log("Installing basic OS tools ..")
    base_install='/usr/bin/dnf -y install '+base_packages
    log(base_install)
    os.system(base_install)
    # common.cmd_live(base_install,log)
    eenosd_python(log)
    eenoscore(log)
    eenos_pip(log)    
    eenos_web(log)
    eenos_dbs(log)
    eenosboot.stage_three(os_data,configure,log)
    eenos_bfd(log)
    eenos_default_html(log)
    eenos_jailshell(log)
    eenos_diskusage(log)
    eenos_monitor(log)
    eenos_networking(log)
    eenos_taskd(log)
    eenos_traffic(log)
    eenos_update(log)
   
