import os
from pprint import pprint
from . import common,eenosboot
from .shellcolor import scolor
import urllib.request
import sys
import tempfile

def jammy_apt_setup(log):
    if os.path.isfile('/etc/apt/apt.conf.d/99needrestart'):
        try:
            os.unlink('/etc/apt/apt.conf.d/99needrestart')
        except Exception as e:
            log(str(e))
            pass    
    try:
        urllib.request.urlretrieve("https://apt.eenos.com/eenos-jammy.list","/etc/apt/sources.list.d/eenos-jammy.list")
        log("Setting up eenos apt repository ")
        log("Setting eenos GPG Keys ..")
        urllib.request.urlretrieve("https://apt.eenos.com/eenos.gpg","/etc/apt/trusted.gpg.d/eenos.gpg")
    except Exception as e:
        log(str(e))
        log("Unable to setup apt repository")
        sys.exit(1)
    
def eenosd_python(log):
    log("Installing eenos python distribution  - [ %seenosd-python%s ]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenosd-python"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenoscore(log):
    log("Installing eenos core package - [%seenoscore%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenoscore"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_pip(log):
    
    # print(tempfile.gettempdir())
    ctmp=str(tempfile.gettempdir())
    tmdir='/root/tmp'
    if not os.path.isdir('/root/tmp'):
        try:
            os.makedirs('/root/tmp')
        except Exception as e:
            log(str(e))
            pass
    os.environ['TMPDIR']=tmdir
    result=0
    try:
        log("Installing Eenos Pips")
        cmd="/usr/local/eenos/bin/pip3 --no-color --no-cache-dir install --extra-index-url https://pip.eenos.com/release/ eenos"
        log(cmd)
        result=os.system(cmd)
    except Exception as e:
        log(str(e))
        log("Error installing python modules. Please make sure /tmp have execute permission at this time")
        sys.exit(0)
    if result > 0:
        log("Error installing python modules. Please make sure /tmp have execute permission at this time")
        log("Then re-run the installer")
        sys.exit(0)
    os.environ['TMPDIR']=ctmp 
    # common.cmd_live(cmd,log)

def eenos_web(log):
    log("Installing eenos web package - [%seenos-web%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-web-release"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
def eenos_dbs(log):
    log("initializing eenos dbs")
    cmd='/usr/local/eenos/scripts/dbinit'
    os.system(cmd)
def eenos_bfd(log):
    log("Installing eenos brute force detection service - [%seenos-bfd%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-bfd" 
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    log("Initializing bfd db")
    cmd2="/usr/local/eenos/scripts/bfdctl -l"
    log(cmd2)
    os.system(cmd2)
    # common.cmd_live(cmd2,log)
    rcmd='/usr/bin/systemctl restart eenosbfd.service'
    log(rcmd)
    os.system(rcmd)
    # common.cmd_live(rcmd,log)

def eenos_default_html(log):
    log("Setting default html docs")
    cmd="/usr/bin/apt-get -y install eenos-default-html"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def eenos_diskusage(log):
    log("Installing eenos disk usage monitor - [%seenos-diskusage%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-diskusage"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_monitor(log):
    log("Installing eenos service monitor - [%seenos-monitor%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-monitor"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_networking(log):
    log("Installing eenos networking service - [%seenos-network%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-network"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_taskd(log):
    log("Installing eenos taskd service - [%seenos_taskd%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-taskd"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def eenos_traffic(log):
    log("Installing eenos bandwidth monitoring service - [%seenos-traffic%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-traffic"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)

def eenos_update(log):
    log("Installing eenos auto updates - [%seenos-update%s]" %(scolor.green,scolor.end))
    cmd="/usr/bin/apt-get -y install eenos-update"
    log(cmd)
    os.system(cmd)
    # common.cmd_live(cmd,log)
    
def cleanup(log):
    log("Cleaning up unused packages")
    cmd="/usr/bin/apt-get -y --purge remove apache2 postfix php mariadb nginx"
    log(cmd)
    os.system(cmd)

def eenos_jailshell(log):
    log("Installing Eenos JailShell")
    cmd="/usr/bin/apt-get -y install eenos-jailshell"
    log(cmd)
    os.system(cmd)

def start_ubuntu_install(os_data,configure,log):
    if os_data['version']=='jammy':
        jammy_apt_setup(log)
    cleanup(log)
    os.environ["DEBIAN_FRONTEND"]="noninteractive"
    apt_update='/usr/bin/apt update -y'
    log(apt_update)
    os.system(apt_update)
    # common.cmd_live(apt_update,log)
    apt_upgrade='/usr/bin/apt upgrade -y'
    log(apt_upgrade)
    os.system(apt_upgrade)
    # common.cmd_live(apt_upgrade,log)
    base_packages='psmisc net-tools wget curl perl lsb-release bind9-host bind9-dnsutils git build-essential ntp rsync rdate quota cron whois iputils-ping iptables pkgconf sqlite3'
    log("Installing basic OS tools ..")
    base_install='/usr/bin/apt-get install '+base_packages+' -y'    
    log(base_install)
    os.system(base_install)
    # common.cmd_live(base_install,log)
    eenosd_python(log)
    eenoscore(log)
    eenos_pip(log)    
    eenos_web(log)
    eenos_dbs(log)
    eenosboot.stage_three(os_data,configure,log)
    eenos_bfd(log)
    eenos_default_html(log)
    eenos_jailshell(log)
    eenos_diskusage(log)
    eenos_monitor(log)
    eenos_networking(log)
    eenos_taskd(log)
    eenos_traffic(log)
    eenos_update(log)
   
